@@ -263,193 +263,101 @@ SpSubview<eT>::operator=(const Base<eT, T1>& in)
   {
   arma_extra_debug_sigprint();
   
-  // this is a modified version of SpSubview::operator_equ_common(const SpBase)
+  const quasi_unwrap<T1> U(in.get_ref());
   
-  const SpProxy< SpMat<eT> > pa((*this).m);
+  arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
   
-  const unwrap<T1>     b_tmp(in.get_ref());
-  const Mat<eT>&   b = b_tmp.M;
+  if(n_elem == 0)  { return *this; }
   
-  arma_debug_assert_same_size(n_rows, n_cols, b.n_rows, b.n_cols, "insertion into sparse submatrix");
   
-  const uword pa_start_row = (*this).aux_row1;
-  const uword pa_start_col = (*this).aux_col1;
+  // generate matrix B
   
-  const uword pa_end_row = pa_start_row + (*this).n_rows - 1;
-  const uword pa_end_col = pa_start_col + (*this).n_cols - 1;
+  const eT*   U_M_memptr = U.M.memptr();
+  const uword U_M_n_elem = U.M.n_elem;
   
-  const uword pa_n_rows = pa.get_n_rows();
+  uword B_n_nonzero = 0;
   
-  const uword b_n_elem = b.n_elem;
-  const eT*   b_mem    = b.memptr();
-  
-  uword box_count = 0;
-  
-  for(uword i=0; i<b_n_elem; ++i)
+  for(uword i=0; i < U_M_n_elem; ++i)
     {
-    box_count += (b_mem[i] != eT(0)) ? uword(1) : uword(0);
+    B_n_nonzero += (U_M_memptr[i] != eT(0)) ? uword(1) : uword(0);
     }
   
-  SpMat<eT> out(pa.get_n_rows(), pa.get_n_cols());
+  SpMat<eT> B(arma_reserve_indicator(), m.n_rows, m.n_cols, B_n_nonzero);
   
-  const uword alt_count = pa.get_n_nonzero() - (*this).n_nonzero + box_count;
+  uword B_count = 0;
   
-  // Resize memory to correct size.
-  out.mem_resize(alt_count);
-  
-  typename SpProxy< SpMat<eT> >::const_iterator_type x_it  = pa.begin();
-  typename SpProxy< SpMat<eT> >::const_iterator_type x_end = pa.end();
-  
-  uword b_row = 0;
-  uword b_col = 0;
-    
-  bool x_it_ok = (x_it != x_end);
-  bool y_it_ok = ( (b_row < b.n_rows) && (b_col < b.n_cols) );
-  
-  uword x_it_row = (x_it_ok) ? x_it.row() : 0;
-  uword x_it_col = (x_it_ok) ? x_it.col() : 0;
-  
-  uword y_it_row = (y_it_ok) ? b_row + pa_start_row : 0;
-  uword y_it_col = (y_it_ok) ? b_col + pa_start_col : 0;
-    
-  uword cur_val = 0;
-  while(x_it_ok || y_it_ok)
+  for(uword col=0; col < U.M.n_cols; ++col)
+  for(uword row=0; row < U.M.n_rows; ++row)
     {
-    const bool x_inside_box = (x_it_row >= pa_start_row) && (x_it_row <= pa_end_row) && (x_it_col >= pa_start_col) && (x_it_col <= pa_end_col);
-    const bool y_inside_box = (y_it_row >= pa_start_row) && (y_it_row <= pa_end_row) && (y_it_col >= pa_start_col) && (y_it_col <= pa_end_col);
+    const eT val = (*U_M_memptr);  U_M_memptr++;
     
-    const eT x_val = x_inside_box ? eT(0) : ( x_it_ok ? (*x_it) : eT(0) );
-    
-    const eT y_val = y_inside_box ? ( y_it_ok ? b.at(b_row,b_col) : eT(0) ) : eT(0);
-    
-    if( (x_it_row == y_it_row) && (x_it_col == y_it_col) )
+    if(val != eT(0))
       {
-      if( (x_val != eT(0)) || (y_val != eT(0)) )  
-        {
-        access::rw(out.values[cur_val]) = (x_val != eT(0)) ? x_val : y_val;
-        access::rw(out.row_indices[cur_val]) = x_it_row;
-        ++access::rw(out.col_ptrs[x_it_col + 1]);
-        ++cur_val;
-        }
-      
-      if(x_it_ok)
-        {
-        ++x_it;
-        
-        if(x_it == x_end)  { x_it_ok = false; }
-        }
-      
-      if(x_it_ok)
-        {
-        x_it_row = x_it.row();
-        x_it_col = x_it.col();
-        }
-      else
-        {
-        x_it_row++;
-        
-        if(x_it_row >= pa_n_rows)  { x_it_row = 0; x_it_col++; }
-        }
-      
-      if(y_it_ok)
-        {
-        b_row++;
-        
-        if(b_row >= b.n_rows)  { b_row = 0; b_col++; }
-        
-        if( (b_row > b.n_rows) || (b_col > b.n_cols) )  { y_it_ok = false; }
-        }
-      
-      if(y_it_ok)
-        {
-        y_it_row = b_row + pa_start_row;
-        y_it_col = b_col + pa_start_col;
-        }
-      else
-        {
-        y_it_row++;
-        
-        if(y_it_row >= pa_n_rows)  { y_it_row = 0; y_it_col++; }
-        }
-      }
-    else
-      {
-      if((x_it_col < y_it_col) || ((x_it_col == y_it_col) && (x_it_row < y_it_row))) // if y is closer to the end
-        {
-        if(x_val != eT(0))
-          {
-          access::rw(out.values[cur_val]) = x_val;
-          access::rw(out.row_indices[cur_val]) = x_it_row;
-          ++access::rw(out.col_ptrs[x_it_col + 1]);
-          ++cur_val;
-          }
-        
-        if(x_it_ok)
-          {
-          ++x_it;
-          
-          if(x_it == x_end)  { x_it_ok = false; }
-          }
-        
-        if(x_it_ok)
-          {
-          x_it_row = x_it.row();
-          x_it_col = x_it.col();
-          }
-        else
-          {
-          x_it_row++;
-          
-          if(x_it_row >= pa_n_rows)  { x_it_row = 0; x_it_col++; }
-          }
-        }
-      else
-        {
-        if(y_val != eT(0))
-          {
-          access::rw(out.values[cur_val]) = y_val;
-          access::rw(out.row_indices[cur_val]) = y_it_row;
-          ++access::rw(out.col_ptrs[y_it_col + 1]);
-          ++cur_val;
-          }
-        
-        if(y_it_ok)
-          {
-          b_row++;
-          
-          if(b_row >= b.n_rows)  { b_row = 0; b_col++; }
-          
-          if( (b_row > b.n_rows) || (b_col > b.n_cols) )  { y_it_ok = false; }
-          }
-        
-        if(y_it_ok)
-          {
-          y_it_row = b_row + pa_start_row;
-          y_it_col = b_col + pa_start_col;
-          }
-        else
-          {
-          y_it_row++;
-          
-          if(y_it_row >= pa_n_rows)  { y_it_row = 0; y_it_col++; }
-          }
-        }
+      access::rw(B.values[B_count])      = val;
+      access::rw(B.row_indices[B_count]) = row + aux_row1;
+      access::rw(B.col_ptrs[col + aux_col1 + 1])++;
+      ++B_count;
       }
     }
   
-  const uword out_n_cols = out.n_cols;
-  
-  uword* col_ptrs = access::rwp(out.col_ptrs);
-  
-  // Fix column pointers to be cumulative.
-  for(uword c = 1; c <= out_n_cols; ++c)
+  for(uword i=0; i < B.n_cols; ++i)
     {
-    col_ptrs[c] += col_ptrs[c - 1];
+    access::rw(B.col_ptrs[i + 1]) += B.col_ptrs[i];
     }
   
-  access::rw((*this).m).steal_mem(out);
   
-  access::rw(n_nonzero) = box_count;
+  if(n_nonzero == 0)
+    {
+    //// insert into submatrix
+    
+    //access::rw(m) = m + B;
+    spglue_merge::apply(access::rw(m), m, B);
+    }
+  else
+    {
+    // generate matrix A
+    
+    const uword row_start = aux_row1;
+    const uword col_start = aux_col1;
+    
+    const uword row_end   = aux_row1 + n_rows - 1;
+    const uword col_end   = aux_col1 + n_cols - 1;
+    
+    SpMat<eT> A(arma_reserve_indicator(), m.n_rows, m.n_cols, m.n_nonzero - n_nonzero);
+    
+    typename SpMat<eT>::const_iterator m_it     = m.begin();
+    typename SpMat<eT>::const_iterator m_it_end = m.end();
+    
+    uword A_count = 0;
+    
+    for(; m_it != m_it_end; ++m_it)
+      {
+      const uword m_it_row = m_it.row();
+      const uword m_it_col = m_it.col();
+      
+      const bool inside_box = ((m_it_row >= row_start) && (m_it_row <= row_end)) && ((m_it_col >= col_start) && (m_it_col <= col_end));
+      
+      if(inside_box == false)
+        {
+        access::rw(A.values[A_count])      = (*m_it);
+        access::rw(A.row_indices[A_count]) = m_it_row;
+        access::rw(A.col_ptrs[m_it_col + 1])++;
+        ++A_count;
+        }
+      }
+    
+    for(uword i=0; i < A.n_cols; ++i)
+      {
+      access::rw(A.col_ptrs[i + 1]) += A.col_ptrs[i];
+      }
+    
+    //// insert into submatrix
+    
+    //access::rw(m) = A + B;
+    spglue_merge::apply(access::rw(m), A, B);
+    }
+  
+  access::rw(n_nonzero) = B_n_nonzero;
   
   return *this;
   }
@@ -558,188 +466,100 @@ SpSubview<eT>::operator_equ_common(const SpBase<eT, T1>& in)
   {
   arma_extra_debug_sigprint();
   
-  // algorithm:
-  // instead of directly inserting values into the matrix underlying the subview,
-  // create a new matrix by merging the underlying matrix with the input object,
-  // and then replacing the underlying matrix with the created matrix.
-  // 
-  // the merging process requires pretending that the input object
-  // has the same size as the underlying matrix.
-  // while iterating through the elements of the input object,
-  // this requires adjusting the row and column locations of each element,
-  // as well as providing fake zero elements.
-  // in effect there is a proxy for a proxy.
+  const unwrap_spmat<T1> U(in.get_ref());
   
-  
-  const SpProxy< SpMat<eT> > pa((*this).m   );
-  const SpProxy< T1        > pb(in.get_ref());
-  
-  arma_debug_assert_same_size(n_rows, n_cols, pb.get_n_rows(), pb.get_n_cols(), "insertion into sparse submatrix");
-  
-  const uword pa_start_row = (*this).aux_row1;
-  const uword pa_start_col = (*this).aux_col1;
-  
-  const uword pa_end_row = pa_start_row + (*this).n_rows - 1;
-  const uword pa_end_col = pa_start_col + (*this).n_cols - 1;
-  
-  const uword pa_n_rows = pa.get_n_rows();
-  
-  SpMat<eT> out(pa.get_n_rows(), pa.get_n_cols());
-  
-  const uword alt_count = pa.get_n_nonzero() - (*this).n_nonzero + pb.get_n_nonzero();
-  
-  // Resize memory to correct size.
-  out.mem_resize(alt_count);
-  
-  typename SpProxy< SpMat<eT> >::const_iterator_type x_it  = pa.begin();
-  typename SpProxy< SpMat<eT> >::const_iterator_type x_end = pa.end();
-  
-  typename SpProxy<T1>::const_iterator_type y_it  = pb.begin();
-  typename SpProxy<T1>::const_iterator_type y_end = pb.end();
-  
-  bool x_it_ok = (x_it != x_end);
-  bool y_it_ok = (y_it != y_end);
-  
-  uword x_it_row = (x_it_ok) ? x_it.row() : 0;
-  uword x_it_col = (x_it_ok) ? x_it.col() : 0;
-  
-  uword y_it_row = (y_it_ok) ? y_it.row() + pa_start_row : 0;
-  uword y_it_col = (y_it_ok) ? y_it.col() + pa_start_col : 0;
-    
-  uword cur_val = 0;
-  while(x_it_ok || y_it_ok)
+  if(U.is_alias(m))
     {
-    const bool x_inside_box = (x_it_row >= pa_start_row) && (x_it_row <= pa_end_row) && (x_it_col >= pa_start_col) && (x_it_col <= pa_end_col);
-    const bool y_inside_box = (y_it_row >= pa_start_row) && (y_it_row <= pa_end_row) && (y_it_col >= pa_start_col) && (y_it_col <= pa_end_col);
+    const SpMat<eT> tmp(U.M);
     
-    const eT x_val = x_inside_box ? eT(0) : ( x_it_ok ? (*x_it) : eT(0) );
-    
-    const eT y_val = y_inside_box ? ( y_it_ok ? (*y_it) : eT(0) ) : eT(0);
-    
-    if( (x_it_row == y_it_row) && (x_it_col == y_it_col) )
-      {
-      if( (x_val != eT(0)) || (y_val != eT(0)) )  
-        {
-        access::rw(out.values[cur_val]) = (x_val != eT(0)) ? x_val : y_val;
-        access::rw(out.row_indices[cur_val]) = x_it_row;
-        ++access::rw(out.col_ptrs[x_it_col + 1]);
-        ++cur_val;
-        }
-      
-      if(x_it_ok)
-        {
-        ++x_it;
-        
-        if(x_it == x_end)  { x_it_ok = false; }
-        }
-      
-      if(x_it_ok)
-        {
-        x_it_row = x_it.row();
-        x_it_col = x_it.col();
-        }
-      else
-        {
-        x_it_row++;
-        
-        if(x_it_row >= pa_n_rows)  { x_it_row = 0; x_it_col++; }
-        }
-      
-      if(y_it_ok)
-        {
-        ++y_it;
-        
-        if(y_it == y_end)  { y_it_ok = false; }
-        }
-      
-      if(y_it_ok)
-        {
-        y_it_row = y_it.row() + pa_start_row;
-        y_it_col = y_it.col() + pa_start_col;
-        }
-      else
-        {
-        y_it_row++;
-        
-        if(y_it_row >= pa_n_rows)  { y_it_row = 0; y_it_col++; }
-        }
-      }
-    else
-      {
-      if((x_it_col < y_it_col) || ((x_it_col == y_it_col) && (x_it_row < y_it_row))) // if y is closer to the end
-        {
-        if(x_val != eT(0))
-          {
-          access::rw(out.values[cur_val]) = x_val;
-          access::rw(out.row_indices[cur_val]) = x_it_row;
-          ++access::rw(out.col_ptrs[x_it_col + 1]);
-          ++cur_val;
-          }
-        
-        if(x_it_ok)
-          {
-          ++x_it;
-          
-          if(x_it == x_end)  { x_it_ok = false; }
-          }
-        
-        if(x_it_ok)
-          {
-          x_it_row = x_it.row();
-          x_it_col = x_it.col();
-          }
-        else
-          {
-          x_it_row++;
-          
-          if(x_it_row >= pa_n_rows)  { x_it_row = 0; x_it_col++; }
-          }
-        }
-      else
-        {
-        if(y_val != eT(0))
-          {
-          access::rw(out.values[cur_val]) = y_val;
-          access::rw(out.row_indices[cur_val]) = y_it_row;
-          ++access::rw(out.col_ptrs[y_it_col + 1]);
-          ++cur_val;
-          }
-        
-        if(y_it_ok)
-          {
-          ++y_it;
-          
-          if(y_it == y_end)  { y_it_ok = false; }
-          }
-        
-        if(y_it_ok)
-          {
-          y_it_row = y_it.row() + pa_start_row;
-          y_it_col = y_it.col() + pa_start_col;
-          }
-        else
-          {
-          y_it_row++;
-          
-          if(y_it_row >= pa_n_rows)  { y_it_row = 0; y_it_col++; }
-          }
-        }
-      }
+    return (*this).operator_equ_common(tmp);
     }
   
-  const uword out_n_cols = out.n_cols;
+  arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
   
-  uword* col_ptrs = access::rwp(out.col_ptrs);
+  if(n_elem == 0)  { return *this; }
   
-  // Fix column pointers to be cumulative.
-  for(uword c = 1; c <= out_n_cols; ++c)
+  
+  // generate matrix B
+  
+  const uword U_M_n_nonzero = U.M.n_nonzero;
+  
+  SpMat<eT> B(arma_reserve_indicator(), m.n_rows, m.n_cols, U_M_n_nonzero);
+  
+  typename SpMat<eT>::const_iterator U_it = U.M.begin();
+  
+  for(uword i=0; i < U_M_n_nonzero; ++i)
     {
-    col_ptrs[c] += col_ptrs[c - 1];
+    access::rw(B.row_indices[i]) = U_it.row() + aux_row1;
+    access::rw(B.col_ptrs[U_it.col() + aux_col1 + 1])++;
+    ++U_it;
     }
   
-  access::rw((*this).m).steal_mem(out);
+  for(uword i=0; i < B.n_cols; ++i)
+    {
+    access::rw(B.col_ptrs[i + 1]) += B.col_ptrs[i];
+    }
   
-  access::rw(n_nonzero) = pb.get_n_nonzero();
+  const eT* B_values_orig = B.values;
+  
+  access::rw(B.values) = U.M.values;  // copy pointer instead of the entire array
+  
+  
+  if(n_nonzero == 0)
+    {
+    //// insert into submatrix
+    
+    //access::rw(m) = m + B;
+    spglue_merge::apply(access::rw(m), m, B);
+    }
+  else
+    {
+    // generate matrix A
+    
+    const uword row_start = aux_row1;
+    const uword col_start = aux_col1;
+    
+    const uword row_end   = aux_row1 + n_rows - 1;
+    const uword col_end   = aux_col1 + n_cols - 1;
+    
+    SpMat<eT> A(arma_reserve_indicator(), m.n_rows, m.n_cols, m.n_nonzero - n_nonzero);
+    
+    typename SpMat<eT>::const_iterator m_it     = m.begin();
+    typename SpMat<eT>::const_iterator m_it_end = m.end();
+    
+    uword A_count = 0;
+    
+    for(; m_it != m_it_end; ++m_it)
+      {
+      const uword m_it_row = m_it.row();
+      const uword m_it_col = m_it.col();
+      
+      const bool inside_box = ((m_it_row >= row_start) && (m_it_row <= row_end)) && ((m_it_col >= col_start) && (m_it_col <= col_end));
+      
+      if(inside_box == false)
+        {
+        access::rw(A.values[A_count])      = (*m_it);
+        access::rw(A.row_indices[A_count]) = m_it_row;
+        access::rw(A.col_ptrs[m_it_col + 1])++;
+        ++A_count;
+        }
+      }
+    
+    for(uword i=0; i < A.n_cols; ++i)
+      {
+      access::rw(A.col_ptrs[i + 1]) += A.col_ptrs[i];
+      }
+    
+    //// insert into submatrix
+    
+    //access::rw(m) = A + B;
+    spglue_merge::apply(access::rw(m), A, B);
+    }
+  
+  
+  access::rw(B.values) = B_values_orig;  // restore original pointer
+  
+  access::rw(n_nonzero) = U_M_n_nonzero;
   
   return *this;
   }
@@ -834,6 +654,159 @@ SpSubview<eT>::operator/=(const SpBase<eT, T1>& x)
 
 
 
+//! apply a functor to each element
+template<typename eT>
+template<typename functor>
+inline
+void
+SpSubview<eT>::for_each(functor F)
+  {
+  arma_extra_debug_sigprint();
+  
+  m.sync_csc();
+  m.invalidate_cache();
+  
+  const uword lstart_row = aux_row1;
+  const uword lend_row   = aux_row1 + n_rows;
+  
+  const uword lstart_col = aux_col1;
+  const uword lend_col   = aux_col1 + n_cols;
+  
+  const uword* m_row_indices = m.row_indices;
+        eT*    m_values      = access::rwp(m.values);
+  
+  bool has_zero = false;
+  
+  for(uword c = lstart_col; c < lend_col; ++c)
+    {
+    const uword r_start = m.col_ptrs[c    ];
+    const uword r_end   = m.col_ptrs[c + 1];
+    
+    for(uword r = r_start; r < r_end; ++r)
+      {
+      const uword m_row_indices_r = m_row_indices[r];
+      
+      if( (m_row_indices_r >= lstart_row) && (m_row_indices_r < lend_row) )
+        {
+        eT& m_values_r = m_values[r];
+        
+        F(m_values_r);
+        
+        if(m_values_r == eT(0))  { has_zero = true; }
+        }
+      }
+    }
+  
+  if(has_zero)
+    {
+    const uword old_m_n_nonzero = m.n_nonzero;
+    
+    access::rw(m).remove_zeros();
+    
+    if(m.n_nonzero != old_m_n_nonzero)
+      {
+      access::rw(n_nonzero) = n_nonzero - (old_m_n_nonzero - m.n_nonzero); 
+      }
+    }
+  }
+
+
+
+template<typename eT>
+template<typename functor>
+inline
+void
+SpSubview<eT>::for_each(functor F) const
+  {
+  arma_extra_debug_sigprint();
+  
+  m.sync_csc();
+  
+  const uword lstart_row = aux_row1;
+  const uword lend_row   = aux_row1 + n_rows;
+  
+  const uword lstart_col = aux_col1;
+  const uword lend_col   = aux_col1 + n_cols;
+  
+  const uword* m_row_indices = m.row_indices;
+  
+  for(uword c = lstart_col; c < lend_col; ++c)
+    {
+    const uword r_start = m.col_ptrs[c    ];
+    const uword r_end   = m.col_ptrs[c + 1];
+    
+    for(uword r = r_start; r < r_end; ++r)
+      {
+      const uword m_row_indices_r = m_row_indices[r];
+      
+      if( (m_row_indices_r >= lstart_row) && (m_row_indices_r < lend_row) )
+        {
+        F(m.values[r]);
+        }
+      }
+    }
+  }
+
+
+
+//! transform each element using a functor
+template<typename eT>
+template<typename functor>
+inline
+void
+SpSubview<eT>::transform(functor F)
+  {
+  arma_extra_debug_sigprint();
+  
+  m.sync_csc();
+  m.invalidate_cache();
+  
+  const uword lstart_row = aux_row1;
+  const uword lend_row   = aux_row1 + n_rows;
+  
+  const uword lstart_col = aux_col1;
+  const uword lend_col   = aux_col1 + n_cols;
+  
+  const uword* m_row_indices = m.row_indices;
+        eT*    m_values      = access::rwp(m.values);
+  
+  bool has_zero = false;
+  
+  for(uword c = lstart_col; c < lend_col; ++c)
+    {
+    const uword r_start = m.col_ptrs[c    ];
+    const uword r_end   = m.col_ptrs[c + 1];
+    
+    for(uword r = r_start; r < r_end; ++r)
+      {
+      const uword m_row_indices_r = m_row_indices[r];
+      
+      if( (m_row_indices_r >= lstart_row) && (m_row_indices_r < lend_row) )
+        {
+        eT& m_values_r = m_values[r];
+        
+        m_values_r = eT( F(m_values_r) );
+        
+        if(m_values_r == eT(0))  { has_zero = true; }
+        }
+      }
+    }
+  
+  if(has_zero)
+    {
+    const uword old_m_n_nonzero = m.n_nonzero;
+    
+    access::rw(m).remove_zeros();
+    
+    if(m.n_nonzero != old_m_n_nonzero)
+      {
+      access::rw(n_nonzero) = n_nonzero - (old_m_n_nonzero - m.n_nonzero); 
+      }
+    }
+  }
+
+
+
 template<typename eT>
 inline
 void
@@ -979,7 +952,7 @@ SpSubview<eT>::eye()
 template<typename eT>
 arma_hot
 inline
-MapMat_svel<eT>
+SpSubview_MapMat_val<eT>
 SpSubview<eT>::operator[](const uword i)
   {
   const uword lrow = i % n_rows;
@@ -1007,7 +980,7 @@ SpSubview<eT>::operator[](const uword i) const
 template<typename eT>
 arma_hot
 inline
-MapMat_svel<eT>
+SpSubview_MapMat_val<eT>
 SpSubview<eT>::operator()(const uword i)
   {
   arma_debug_check( (i >= n_elem), "SpSubview::operator(): index out of bounds");
@@ -1039,7 +1012,7 @@ SpSubview<eT>::operator()(const uword i) const
 template<typename eT>
 arma_hot
 inline
-MapMat_svel<eT>
+SpSubview_MapMat_val<eT>
 SpSubview<eT>::operator()(const uword in_row, const uword in_col)
   {
   arma_debug_check( (in_row >= n_rows) || (in_col >= n_cols), "SpSubview::operator(): index out of bounds");
@@ -1065,7 +1038,7 @@ SpSubview<eT>::operator()(const uword in_row, const uword in_col) const
 template<typename eT>
 arma_hot
 inline
-MapMat_svel<eT>
+SpSubview_MapMat_val<eT>
 SpSubview<eT>::at(const uword i)
   {
   const uword lrow = i % n_rows;
@@ -1093,12 +1066,10 @@ SpSubview<eT>::at(const uword i) const
 template<typename eT>
 arma_hot
 inline
-MapMat_svel<eT>
+SpSubview_MapMat_val<eT>
 SpSubview<eT>::at(const uword in_row, const uword in_col)
   {
-  m.sync_cache();
-  
-  return m.cache.svel(aux_row1 + in_row, aux_col1 + in_col, m.sync_state, access::rw(m.n_nonzero), access::rw(n_nonzero));
+  return SpSubview_MapMat_val<eT>((*this), m.cache, aux_row1 + in_row, aux_col1 + in_col);
   }
 
 
@@ -1525,6 +1496,18 @@ SpSubview<eT>::begin() const
 
 
 
+template<typename eT>
+inline
+typename SpSubview<eT>::const_iterator
+SpSubview<eT>::cbegin() const
+  {
+  m.sync_csc();
+  
+  return const_iterator(*this);
+  }
+
+
+
 template<typename eT>
 inline
 typename SpSubview<eT>::iterator
@@ -1596,6 +1579,18 @@ SpSubview<eT>::end() const
 
 
 
+template<typename eT>
+inline
+typename SpSubview<eT>::const_iterator
+SpSubview<eT>::cend() const
+  {
+  m.sync_csc();
+  
+  return const_iterator(*this, 0, n_cols, n_nonzero, m.n_nonzero - n_nonzero);
+  }
+
+
+
 template<typename eT>
 inline
 typename SpSubview<eT>::row_iterator
@@ -1644,6 +1639,16 @@ SpSubview<eT>::end_row(const uword row_num) const
 
 
 
+template<typename eT>
+arma_inline
+bool
+SpSubview<eT>::is_alias(const SpMat<eT>& X) const
+  {
+  return m.is_alias(X);
+  }
+
+
+
 template<typename eT>
 inline
 arma_warn_unused
